﻿/*
 * 
 * Copyright (c) 2015, H.R.Z. SOftware Services LTD
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions 
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer 
 *    in the documentation and/or other materials provided with the distribution.
 * 
 * 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from 
 *    this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

/// \page CSTestApplications C# Test Applications
/// The C# test applications can be downloaded from www.roniza.com/downloads
/// \section CheckDICOM Basic RZDCX Example
/// This example demonstrates the parsing of DICOM files from the selected folder.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using rzdcxLib;

namespace CheckDICOM
{
    class Program
    {
        static void Main(string[] args)
        {
            Console.WriteLine("Enter folder:"); // Prompt
            string line = Console.ReadLine(); // Get string from user

            if (Directory.Exists(line))
            {
                FileStream filestream = new FileStream("out.csv", FileMode.Create);
                StreamWriter streamwriter = new StreamWriter(filestream);
                streamwriter.AutoFlush = true;
                Console.SetOut(streamwriter);


                Console.WriteLine("File Name, PatientID, PatientName, PatientBirthDate, PatientSex, StudyInstanceUID, SeriesInstanceUID, SOPInstanceUID");

                HandleFolder(line);
            }

            StreamWriter standardOutput = new StreamWriter(Console.OpenStandardOutput());
            standardOutput.AutoFlush = true;
            Console.SetOut(standardOutput);

            Console.WriteLine("Press any key to exit:"); // Prompt
            Console.ReadLine();
        }

        static private void HandleFolder(string InputFolder)
        {
            string[] files = Directory.GetFiles(InputFolder);

            DCXOBJ obj = null;

            DCXELM e = null;
            string tmp = "";
            foreach (string file in files)
            {
                tmp = file;
                try
                {
                    obj = new DCXOBJ();
                    obj.openFile(file);
                    tmp += @",";
                    try
                    {
                        e = obj.getElementByTag((int)DICOM_TAGS_ENUM.patientID);
                        tmp += e.Value.ToString();
                    }
                    catch (Exception)
                    {
                    }
                    tmp += @",";
                    try
                    {
                        e = obj.getElementByTag((int)DICOM_TAGS_ENUM.patientName);
                        tmp += e.Value.ToString();
                    }
                    catch (Exception)
                    {
                    }
                    tmp += @",";
                    try
                    {
                        e = obj.getElementByTag((int)DICOM_TAGS_ENUM.PatientBirthDate);
                        tmp += e.Value.ToString();
                    }
                    catch (Exception)
                    {
                    }
                    tmp += @",";
                    try
                    {
                        e = obj.getElementByTag((int)DICOM_TAGS_ENUM.PatientSex);
                        tmp += e.Value.ToString();
                    }
                    catch (Exception)
                    {
                    }
                    tmp += @",";
                    try
                    {
                        e = obj.getElementByTag((int)DICOM_TAGS_ENUM.studyInstanceUID);
                        tmp += e.Value.ToString();
                    }
                    catch (Exception)
                    {
                    }
                    tmp += @",";
                    try
                    {
                        e = obj.getElementByTag((int)DICOM_TAGS_ENUM.seriesInstanceUID);
                        tmp += e.Value.ToString();
                    }
                    catch (Exception)
                    {
                    }
                    tmp += @",";
                    try
                    {
                        e = obj.getElementByTag((int)DICOM_TAGS_ENUM.sopInstanceUID);
                        tmp += e.Value.ToString();
                    }
                    catch (Exception)
                    {
                    }
                }
                catch (Exception ex)
                {
                    tmp += " Error: ";
                    tmp += ex.Message;
                }
                Console.WriteLine(tmp);
            }

            string[] dirs = Directory.GetDirectories(InputFolder);
            foreach (string dir in dirs)
            {
                HandleFolder(dir);
            }
        }
    }


}
